/*
  lp_BLE_TempCalibration
  (c)2022 Forward Computing and Control Pty. Ltd.
   This code is not warranted to be fit for any purpose. You may only use it at your own risk.
   This code may be freely used for both private and commercial use
   Provide this copyright is maintained.
*/

const char LOCAL_NAME[] = "T_0";
//  say T_.. for Temp(C) sensors, H_.. for Temp(C),Humditiy(%) sensors, W_.. for Temp(C),Humdity(%),Barometer(hPa) sensors, WL_.. for Temp(C),Humdity(%),Barometer(hPa),Lux level(Lux)
//  keep this LOCAL_NAME short as it uses up available advertising data space (see below)
// Format of advertized name
// For T_.. devices   LOCAL_NAME,degC
// For H_.. devices   LOCAL_NAME,degC,%RH
// For W_.. devices   LOCAL_NAME,degC,%RH,hPa
// For WL_.. devices  LOCAL_NAME,degC,%RH,hPa,Lux
#include <Wire.h>
#include "lp_Si7021.h"
#include "SafeString.h"
#include "BLEPeripheral.h"

BLEPeripheral ble;

// BLE_SCAN_DATA_MAX_VALUE_LENGTH == 29 in BLEDeviceLimits.h
// to this is added length and type for total of 31 == max advertisize packet size
// here we also set flags for not connectable which uses 3 bytes length(1), type(1), flags(1)
// so MAX_SCAN_DATA_LEN reduced to 26
const size_t MAX_DATA_LEN = 26;
// this can be split between say device NAME and Manufacturing Data
// but that uses another two bytes for the length and type bytes for the Manufacturing Data
//
// For simplicity and readability here all the data is added to the NAME as ascii date comma separated
// If using NAME and  Manufacturing Data then sending data as binary in  Manufacturing Data allows for more data.
//
// Note could also just have  Manfacturing Data with out a device NAME but easier to read data in the name with scanning apps

// uncomment this to enable Serial output uses a lot of supply current
//#define DEBUG

cSF(sfData, MAX_DATA_LEN); // Local name and formatted measurements SafeString will limit data to this length. Excess data is dropped NOT truncated see SafeString docs/tutorials

unsigned short ADVERT_INTERVAL = 750; // default it 500, 750ms gives 13 advert packets in the 10 secs

lp_timer BLE_Timer; // the advert timer

// 10sec advert on 90sec advert off
//const unsigned long BLE_ADVERT_MS = 10ul * 1000; // advertise for 10sec at 750ms intervals i.e. ~13 advert packets
//const unsigned long BLE_NO_ADVERT_MS = 90ul * 1000; // go silent for 90sec

// 10sec advert on 20sec advert off
const unsigned long BLE_ADVERT_MS = 10ul * 1000; // advertise for 10sec at 750ms intervals i.e. ~13 advert packets
const unsigned long BLE_NO_ADVERT_MS = 20ul * 1000; // go silent for 20sec
float chipTemp = 0.0;

uint16_t tempCRaw = 0;
uint16_t humidityRaw = 0;

float tempCompensation(float reading) {
  return reading;
}
bool sensorIntialized = false; // if not true do not read values
char *errorStr = ""; // sensor initialization error msg, if any.

void startRHandTempConversion();
void readRHResults(); // this restarts ble also

lp_Si7021 sensor;
lp_timer readResultDelay;
const unsigned long READ_RESULT_DELAY_TIME = 25; // ms

void setAdvertData() {
  sfData = LOCAL_NAME;
  sfData += ',';
  float sensorTemp = sensor.Temp_RawToFloat(tempCRaw);
  sfData.print(sensor.Temp_RawToFloat(tempCRaw));
  sfData += ',';
  chipTemp =  getChipTemperature(); // -40C to 85C in 0.25degC steps
  chipTemp = tempCompensation(chipTemp); // add corrections
  sfData.print(chipTemp - sensorTemp);
  ble.setAdvertisedName(sfData.c_str());  //
#ifdef DEBUG
  Serial.print("set data to ");
  Serial.println(sfData);
#endif

}

void setup() {
#ifdef DEBUG
  Serial.setPins(30, 29); // Rx P0.30, Tx P0.29, the defaults
  Serial.begin(115200);
  Serial.println();
  for (int i = 10; i > 0; i--) {
    Serial.print(' '); Serial.print(i);
    delay(500);
  }
  Serial.println();
#endif

  Wire.setPins(28, 31); // sda 28 = D2, clk 31 = D3

  if (sensor.begin()) {
#ifdef DEBUG
    Serial.println("Found Si7021");
#endif
    sensorIntialized = true; // else leave as false
  } else {
    errorStr = "Error\n Si7021 not found";
  }
#ifdef DEBUG
  Serial.println(errorStr);
#endif
  // set advertised name

  ble.setConnectable(false);
  ble.setTxPower(+4);
  ble.setAdvertisingInterval(ADVERT_INTERVAL);

  startRHandTempConversion(); // do first conversion

  // setAdvertData(); // sets name and measurement data
  ble.begin();
#ifdef DEBUG
  Serial.println(" BLE started");
#endif
  // BLE_Timer.startDelay(BLE_ADVERT_MS * 3, stopAdvert); // initially advertise for 30sec to make it easier to check the unit is running
#ifdef DEBUG
  Serial.println("setup() finished");
#endif
}

void stopAdvert() {
  ble.end();
#ifdef DEBUG
  Serial.print("stop ble "); Serial.println(millis());
#endif
  BLE_Timer.startDelay(BLE_NO_ADVERT_MS, startRHandTempConversion); //wait 20sec before restarting
}

void restartAdvert() {
  setAdvertData();  // pick up latest values
  ble.begin();
#ifdef DEBUG
  Serial.print("start ble ");; Serial.println(millis());
#endif
  BLE_Timer.startDelay(BLE_ADVERT_MS, stopAdvert); // stop in 10sec
}

void loop() {
  sleep(); // just sleep here waiting for the timer to trigger
}

bool conversionRunning = false;
void startRHandTempConversion() {
  if (!conversionRunning) {
    conversionRunning = true;
    if (sensorIntialized) {
      sensor.startRHandTemp();
    }
  } // else just schedule to pickup last result
  readResultDelay.startDelay(READ_RESULT_DELAY_TIME, readRHResults);
}

void readRHResults() {
  if (sensorIntialized) {
    // Get Result of Relative Humidity from or Si7021
    humidityRaw = sensor.readRH_Raw();
    // Read Result of Temperature from RH measurement Si7021
    tempCRaw = sensor.readLastRHTemp_Raw();
  }
  conversionRunning = false;
  restartAdvert(); // with new values read above
#ifdef DEBUG
  Serial.print("Chip Temp:");  Serial.print(chipTemp);  Serial.print("C, ");
  Serial.println();
  Serial.print("Temp:");  Serial.print(sensor.Temp_RawToFloat(tempCRaw));  Serial.print("C, ");
  Serial.print("Humidity:");  Serial.print(sensor.RH_RawToInt(humidityRaw));  Serial.print("%");
  Serial.println();
#endif
}
